;
; Usage: ENCRYPT InputFile OutputFile
;
; Prompt for a password, then read InputFile and encrypt it with the
; password, writing the result to OutputFile.
;
; Append the result from this file to PASSLDR.COM.
; e.g. copy /B PASSLDR.COM+OUTPUT.COM RUNME.COM
;
; Copyright           ztank August 1998
;

.model tiny

code		segment
		assume cs:code, ds:code
		org 100h

		
prog_start:
		cld

		; parse arguments
		mov	si, offset msg_usage
		mov	ax, 1
		mov	di, offset fname
		call	GetArg
		jc	error_msg
		mov	ax, 2
		mov	di, offset enc_fname
		call	GetArg
		jc	error_msg

		; first read in file to encrypt
		mov	ax, 03d00h			; open file for reading
		mov	dx, offset fname
		int	21h
		mov	si, offset msg_openerror
		jc	error_msg
		mov	bx, ax

		; read data
		mov	word ptr [num_bytes], 0
keep_reading:
		mov	ah, 3fh				; read bytes from file
		mov	cx, 01000h			; in 4K chunks
		mov	dx, offset data
		add	dx, word ptr [num_bytes]	; next offset to write data
		int	21h
		mov	si, offset msg_readerror
		jc	error_msg
		add	word ptr [num_bytes], ax	; update number of bytes
		test	ax, ax				; 0 bytes = EOF
		jnz	keep_reading

		; close the file
		mov	ah, 3eh
		int	21h

		; get the password from the user
		mov	si, offset prompt
		call	PrintStr

		call	ReadPasswd
		mov	word ptr [checksum], cx

		; now to encrypt the rest of the code using ax, bx
		mov	cx, word ptr [num_bytes]
		mov	si, offset data
		call	Encrypt

		; open file for output
		mov	ax, 03c00h			; create file
		xor	cx, cx				; no attribs
		mov	dx, offset enc_fname
		int	21h
		mov	bx, ax

		; write encrypted data
		mov	ah, 40h				; write bytes
		mov	cx, word ptr [num_bytes]
		add	cx, (data - encrypt_block)	; size of header
		mov	dx, offset encrypt_block
		int	21h
		mov	si, offset msg_writeerror
		jc	error_msg

		; close the file
		mov	ah, 3eh
		int	21h

prog_end:
		mov	ax, 4c00h
		int	21h				; end

error_msg:
		call	PrintStr
		mov	ax, 4c01h
		int	21
		
;------------------------------------------------------------------------------

include		GetArg.inc
include		PrintStr.inc
include		ReadPass.inc
include		Encrypt.inc

;-----------------------------------------------------------------------------

prompt		db "Enter password:",0
msg_openerror	db "Error opening "
fname		db 20 dup (?)
enc_fname	db 20 dup (?)
msg_readerror	db "Error reading",10,13,0
msg_writeerror	db "Error writing",10,13,0
msg_usage	db "Usage: encrypt <COM file> <output file>",13,10
		db "will make an encrypted version of <COM file> called <out file>",13,10,0

encrypt_block	equ $
checksum	equ $ + 0
num_bytes	equ $ + 2
data		equ $ + 4

code		ends

end prog_start
