;
; Usage: FFMAKE InputFile OutputFile
;
; Combine [ffloader.com] + [size of InputFile] + [XORed (0xff) InputFile] --> [OutputFile]
; Use with FFLOADER.ASM.
;
; Copyright           ztank August 1998
;

.model tiny

code		segment
		assume cs:code, ds:code
		org 100h
prog_start:
		cld
		mov bp, offset data

		; first parse the arguments
		mov	si, offset msg_usage
		mov	ax, 1
		mov	di, offset prog_name
		call	GetArg
		jc	error_msg
		mov	ax, 2
		mov	di, offset out_file
		call	GetArg
		jc	error_msg

		; read the loader
		mov ax, offset loader_name
		mov dx, bp
		call read_file_fully
		jc error

		; store size of loader and
		; make space for the number of bytes entry (2 bytes)
		mov word ptr [loader_size], ax
		inc ax
		inc ax

		; read the program file
		mov dx, bp
		add dx, ax
		mov ax, offset prog_name
		call read_file_fully
		jc error

		; save the size of the program
		mov word ptr [prog_size], ax
		mov si, word ptr [loader_size]
		mov word ptr [bp+si], ax	; store num bytes read

		; encryption time...
		mov si, bp
		add si, word ptr [loader_size]
		inc si
		inc si				; si -> prog bytes
		mov cx, word ptr [prog_size]
next_byte:
		xor byte ptr [si], 0ffh
		inc si
		loop next_byte

		; work out total bytes
		xor ax, ax
		inc ax
		inc ax
		add ax, word ptr [loader_size]
		add ax, word ptr [prog_size]	; ax = total size
		mov cx, ax
		mov dx, bp
		mov ax, offset out_file
		call write_file
		jc error

		mov ax, 4c00h
		int 21h

error:
		mov si, offset msg_error
error_msg:
		call PrintStr
		mov ax, 4c01h
		int 21h

;-----------------------------------------------------------------------------

include		GetArg.inc
include		PrintStr.inc
		
;-----------------------------------------------------------------------------

; Read the file pointed to by ax, to the buffer dx until EOF.
; The number of bytes read is returned in ax.

read_file_fully	PROC NEAR

		push bx
		push cx
		push dx
		push si
		push di

		mov di, dx			; ptr to buffer

		; open file
		mov dx, ax
		mov ax, 03d00h
		int 21h
		jc read_file_fully_error
		mov bx, ax

		; read data
		xor si, si			; number if bytes read
keep_reading:
		mov ah, 3fh			; read bytes from file
		mov cx, 01000h			; in 4K chunks
		mov dx, di			; start of buffer
		add dx, si			; next offset to write data
		int 21h
		jc read_file_fully_error
		add si, ax			; update number of bytes
		test ax, ax			; 0 bytes = EOF
		jnz keep_reading

		; close the file
		mov ah, 3eh
		int 21h

read_file_fully_exit:
		mov ax, si			; number of bytes read

		pop di
		pop si
		pop dx
		pop cx
		pop bx
		ret

read_file_fully_error:
		; close the file
		mov ah, 3eh
		int 21h
		stc
		jmp read_file_fully_exit

read_file_fully	ENDP

;-----------------------------------------------------------------------------

; Write to the file pointed to by ax (ASCIZ).
; cx bytes are written, from the buffer pointed to by dx.

write_file	PROC NEAR

		push ax
		push bx
		push cx
		push dx

		; open file for output
		mov dx, ax			; file name
		mov ax, 03c00h			; create file
		xor cx, cx			; no attribs
		int 21h
		jc write_file_error
		mov bx, ax

		; write data
		pop dx
		pop cx				; get number of bytes to write
		push cx
		push dx				; get buffer offset
		mov ah, 40h			; write bytes
		int 21h
		jc write_file_error

		; close the file
		mov ah, 3eh
		int 21h

write_file_exit:
		pop dx
		pop cx
		pop bx
		pop ax
		ret

write_file_error:
		; close the file
		mov ah, 3eh
		int 21h
		stc				; set carry flag for error
		jmp write_file_exit

write_file	ENDP

;-----------------------------------------------------------------------------

msg_error	db "Error occurred!",10,13,0
msg_usage	db "Usage: FFMAKER [infile] [outfile]",13,10
		db "will make a version of [infile] which has an XOR-with-0xff loader",13,10,0
loader_name	db "ffloader.com",0
prog_name	db 20 dup (?)
out_file	db 20 dup (?)
loader_size	dw ?
prog_size	dw ?
data		equ $

code		ends

end		prog_start